package web_handlers

import (
	"encoding/json"
	"fmt"
	"net/http"
	"sort"

	"git.agecem.com/agecem/agecem-org/api"
	"git.agecem.com/agecem/agecem-org/models"
	"github.com/labstack/echo/v4"
)

func HandleIndex(c echo.Context) error {
	return c.Render(http.StatusOK, "index-html", nil)
}

/*
func HandleAPropos(c echo.Context) error {
	return c.Render(http.StatusOK, "a-propos-html", nil)
}
*/

/*
func HandleActualite(c echo.Context) error {
	return c.Render(http.StatusOK, "actualite-html", nil)
}
*/

/*
func HandleActualiteArticle(c echo.Context) error {
	article := c.Param("article")
	return c.String(http.StatusOK, fmt.Sprintf("Article: %s", article))
}
*/

func HandleVieEtudiante(c echo.Context) error {
	return c.Render(http.StatusOK, "vie-etudiante-html", nil)
}

func HandleVieEtudianteOrganisme(c echo.Context) error {
	organisme := c.Param("organisme")
	return c.String(http.StatusOK, fmt.Sprintf("Organisme: %s", organisme))
}

func HandleDocumentation(c echo.Context) error {
	client, err := api.NewApiClientFromViper()
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	result, err := client.Call(http.MethodGet, "/v1/bucket")
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	var buckets map[string]string

	err = json.Unmarshal(result, &buckets)
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	var data []models.Bucket

	for bucket, displayName := range buckets {
		content, err := client.Call(http.MethodGet, fmt.Sprintf("/v1/bucket/%s", bucket))
		if err != nil {
			return c.Render(http.StatusInternalServerError, "documentation-html", nil)
		}

		var documents []string

		err = json.Unmarshal(content, &documents)
		if err != nil {
			return c.Render(http.StatusInternalServerError, "documentation-html", nil)
		}

		// Ce bloc retire tous les caractères spéciaux d'une string
		// N'est pas présentement activé, car les fichiers sont processed
		// à la création de toute façon.
		/*
				reg, err := regexp.Compile("[^.a-zA-Z0-9_-]+")
				if err != nil {
					return c.Render(http.StatusInternalServerError, "documentation-html", nil)
				}

				var documents_processed []string

				for _, document := range documents {
					document_processed := reg.ReplaceAllString(document, "")
					documents_processed = append(documents_processed, document_processed)
				}
			documents_processed := documents
		*/

		data = append(data, models.Bucket{
			Name:        bucket,
			DisplayName: displayName,
			Documents:   documents,
		})
	}

	sort.SliceStable(data, func(i, j int) bool { return data[i].Name < data[j].Name })

	return c.Render(http.StatusOK, "documentation-html", data)
}

func HandleFormulaires(c echo.Context) error {
	return c.Render(http.StatusOK, "formulaires-html", nil)
}

func HandlePublicDocumentation(c echo.Context) error {
	client, err := api.NewApiClientFromViper()
	if err != nil {
		return c.JSON(http.StatusNotFound, map[string]string{"message": "Not Found"})
	}

	bucket := c.Param("bucket")
	document := c.Param("document")

	result, err := client.Call(http.MethodGet, fmt.Sprintf("/v1/bucket/%s/%s", bucket, document))
	if err != nil {
		return c.JSON(http.StatusNotFound, map[string]string{"message": "Not Found"})
	}

	// Check if result can fit inside a map containing a message
	var result_map map[string]string

	err = json.Unmarshal(result, &result_map)
	if err == nil {
		return c.JSON(http.StatusBadRequest, result_map)
	}

	return c.Blob(http.StatusOK, "application/octet-stream", result)
}

func HandleAdmin(c echo.Context) error {
	return c.Render(http.StatusOK, "admin-html", nil)
}

func HandleAdminDocumentsUpload(c echo.Context) error {
	client, err := api.NewApiClientFromViper()
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	result, err := client.Call(http.MethodGet, "/v1/bucket")
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	var buckets map[string]string

	err = json.Unmarshal(result, &buckets)
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	var data struct {
		Buckets []models.Bucket
		Message string
	}

	for bucketName, displayName := range buckets {
		data.Buckets = append(data.Buckets, models.Bucket{
			Name:        bucketName,
			DisplayName: displayName,
		})
	}

	return c.Render(http.StatusOK, "admin-upload-html", data)
}

func HandleAdminDocumentsUploadPOST(c echo.Context) error {
	var data struct {
		Buckets []models.Bucket
		Message string
	}

	client, err := api.NewApiClientFromViper()
	if err != nil {
		data.Message = fmt.Sprintf("HandleAdminDocumentsUploadPOST#api.New: %s", err)
		return c.Render(http.StatusInternalServerError, "admin-upload-html", data)
	}

	result, err := client.Call(http.MethodGet, "/v1/bucket")
	if err != nil {
		data.Message = "Error during GET /v1/bucket"
		return c.Render(http.StatusInternalServerError, "documentation-html", data)
	}

	var buckets map[string]string

	err = json.Unmarshal(result, &buckets)
	if err != nil {
		return c.Render(http.StatusInternalServerError, "documentation-html", nil)
	}

	for bucketName, displayName := range buckets {
		data.Buckets = append(data.Buckets, models.Bucket{
			Name:        bucketName,
			DisplayName: displayName,
		})
	}

	bucket := c.FormValue("bucket")

	document, err := c.FormFile("document")
	if err != nil {
		data.Message = fmt.Sprintf("HandleAdminDocumentsUploadPOST#c.FormFile: %s", err)
		return c.Render(http.StatusBadRequest, "admin-upload-html", data)
	}

	response, err := client.UploadDocument(bucket, document)
	if err != nil {
		data.Message = fmt.Sprintf("HandleAdminDocumentsUploadPOST#client.UploadDocument: %s", err)
		return c.Render(http.StatusInternalServerError, "admin-upload-html", data)
	}

	// Format response
	var info, status string

	info = fmt.Sprintf("[%.0f] /public/documentation/%s/%s", response.Data.Size, response.Data.Bucket, response.Data.Object)

	status = response.Message

	data.Message = fmt.Sprintf("%s - %s", status, info)

	return c.Render(http.StatusOK, "admin-upload-html", data)
}
